
# q-expansion of newform 238.2.a.f, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 238
weight = 2
poly_data = [-1, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[-1, 0], [1, 1], [1, -1], [-1, 0], [3, 1], [2, -2], [1, 0], [-4, -2], [8, 0], [1, -3], [-6, 2], [-1, -1], [-8, 2], [-2, 2], [2, -2], [0, -2], [-6, 0], [-1, 1], [-6, 2], [2, -2], [0, 6], [-6, -2], [2, 0], [2, 0], [-4, -2], [-2, -2], [-4, 0], [1, -1], [1, 5], [0, 6], [12, 0], [-5, 3], [8, -2], [-1, -5], [10, 4], [4, 0], [12, 4], [-7, 3], [-10, -2], [5, 7], [0, -4], [-5, 1], [4, 4], [6, -8], [15, 3], [10, 2], [-21, -3], [6, -2], [3, 11], [4, -4], [2, 8], [4, -4], [-6, 8], [14, 0], [-2, 0], [16, -4], [-7, 3], [6, 6], [-7, -7], [8, -2], [17, -3], [-28, 0], [-8, 2], [-12, -4], [8, -6], [-17, -5], [2, -10], [-16, -2], [-11, 7], [-26, 2], [6, -8], [-4, 12], [-20, -4], [0, 6], [7, -11], [0, -4], [26, 0], [7, 5], [-6, 4], [6, 4], [-1, -13], [-20, 2], [-4, -4], [2, 4], [-12, 4], [14, -10], [-4, 6], [-2, 8], [-10, 2], [24, 4], [12, -10], [-6, -6], [12, -4], [-24, -4], [-25, -7], [4, 4], [-12, 4], [-10, 4], [-34, -4], [-21, -1], [17, 11], [12, 10], [-6, -12], [-2, 8], [-11, -1], [30, 0], [8, 6], [6, -4], [-4, 16], [14, 0], [-8, 0], [-18, -12], [-12, 10], [19, 3], [12, -4], [18, -4], [-7, 9], [-2, 10], [17, -11], [-12, -4], [10, -6], [8, -14], [23, 5], [35, 5], [-19, -7], [4, -18], [1, -11], [-14, -6], [-42, 2], [-18, 6], [20, 12], [34, -2], [20, 4], [18, 0], [-14, 8], [22, -12], [4, 0], [3, 3], [26, 6], [10, -4], [-31, 5], [-5, -17], [2, -2], [-13, -11], [32, 4], [30, -10], [-19, -1], [-8, 2], [-18, -8], [8, 4], [-11, 9], [-38, -4], [-10, 6], [-2, -10], [-9, 1], [-10, 10], [-36, 4], [-30, 0], [38, -8], [39, 9], [-27, -9], [14, -4], [36, 4], [-2, 12], [-50, -4], [50, 2], [18, -18], [-5, -19]]
